@@ -26,15 +26,12 @@
 #define TEMPLATES_H_INCLUDED
 
 #include "syslog-ng.h"
-#include "common-template-typedefs.h"
+#include "eval.h"
 #include "timeutils/zoneinfo.h"
-#include "type-hinting.h"
+#include "logmsg/type-hinting.h"
+#include "common-template-typedefs.h"
 #include "atomic.h"
 
-#define LTZ_LOCAL 0
-#define LTZ_SEND  1
-#define LTZ_MAX   2
-
 #define LOG_TEMPLATE_ERROR log_template_error_quark()
 
 GQuark log_template_error_quark(void);
@@ -55,35 +52,26 @@ typedef enum
 } LogTemplateOnError;
 
 /* structure that represents an expandable syslog-ng template */
-typedef struct _LogTemplate
+struct _LogTemplate
 {
   GAtomicCounter ref_cnt;
   gchar *name;
   gchar *template;
   GList *compiled_template;
   GlobalConfig *cfg;
-  guint escape:1, def_inline:1, trivial:1;
-  TypeHint type_hint;
-} LogTemplate;
+  guint escape:1, def_inline:1, trivial:1, literal:1;
 
-/* template expansion options that can be influenced by the user and
- * is static throughout the runtime for a given configuration. There
- * are call-site specific options too, those are specified as
- * arguments to log_template_format() */
-struct _LogTemplateOptions
-{
-  gboolean initialized;
-  /* timestamp format as specified by ts_format() */
-  gint ts_format;
-  /* number of digits in the fraction of a second part, specified using frac_digits() */
-  gint frac_digits;
 
-  /* timezone for LTZ_LOCAL/LTZ_SEND settings */
-  gchar *time_zone[LTZ_MAX];
-  TimeZoneInfo *time_zone_info[LTZ_MAX];
+  /* This value stores the type-hint the user _explicitly_ specified.  If
+   * this is an automatic cast to string (in compat mode), this would be
+   * LM_VT_NONE while "type_hint" would be LM_VT_STRING */
+  LogMessageValueType explicit_type_hint;
 
-  /* Template error handling settings */
-  gint on_error;
+  /* This is the type-cast we do perform as we evaluate this template.  It
+   * might differ from explicit_type_hint in case we are in compatibility
+   * mode where the template would be cast to a string when the type-hint is
+   * unspecified.  */
+  LogMessageValueType type_hint;
 };
 
 /* appends the formatted output into result */
@@ -91,27 +79,27 @@ struct _LogTemplateOptions
 void log_template_set_escape(LogTemplate *self, gboolean enable);
 gboolean log_template_set_type_hint(LogTemplate *self, const gchar *hint, GError **error);
 gboolean log_template_compile(LogTemplate *self, const gchar *template, GError **error);
+gboolean log_template_compile_with_type_hint(LogTemplate *self, const gchar *template_and_typehint, GError **error);
+void log_template_forget_template_string(LogTemplate *self);
 void log_template_compile_literal_string(LogTemplate *self, const gchar *literal);
+gboolean log_template_is_literal_string(const LogTemplate *self);
+const gchar *log_template_get_literal_value(const LogTemplate *self, gssize *value_len);
 gboolean log_template_is_trivial(LogTemplate *self);
+NVHandle log_template_get_trivial_value_handle(LogTemplate *self);
 const gchar *log_template_get_trivial_value(LogTemplate *self, LogMessage *msg, gssize *value_len);
-void log_template_format(LogTemplate *self, LogMessage *lm, const LogTemplateOptions *opts, gint tz, gint32 seq_num,
-                         const gchar *context_id, GString *result);
-void log_template_append_format(LogTemplate *self, LogMessage *lm, const LogTemplateOptions *opts, gint tz,
-                                gint32 seq_num, const gchar *context_id, GString *result);
-void log_template_append_format_with_context(LogTemplate *self, LogMessage **messages, gint num_messages,
-                                             const LogTemplateOptions *opts, gint tz, gint32 seq_num, const gchar *context_id, GString *result);
-void log_template_format_with_context(LogTemplate *self, LogMessage **messages, gint num_messages,
-                                      const LogTemplateOptions *opts, gint tz, gint32 seq_num, const gchar *context_id, GString *result);
+const gchar *log_template_get_trivial_value_and_type(LogTemplate *self, LogMessage *msg, gssize *value_len,
+                                                     LogMessageValueType *type);
 void log_template_set_name(LogTemplate *self, const gchar *name);
 
 LogTemplate *log_template_new(GlobalConfig *cfg, const gchar *name);
 LogTemplate *log_template_ref(LogTemplate *s);
 void log_template_unref(LogTemplate *s);
 
-
+void log_template_options_clone(LogTemplateOptions *source, LogTemplateOptions *dest);
 void log_template_options_init(LogTemplateOptions *options, GlobalConfig *cfg);
 void log_template_options_destroy(LogTemplateOptions *options);
 void log_template_options_defaults(LogTemplateOptions *options);
+void log_template_options_global_defaults(LogTemplateOptions *options);
 
 void log_template_global_init(void);
 void log_template_global_deinit(void);
@@ -119,4 +107,6 @@ void log_template_global_deinit(void);
 gboolean log_template_on_error_parse(const gchar *on_error, gint *out);
 void log_template_options_set_on_error(LogTemplateOptions *options, gint on_error);
 
+EVTTAG *evt_tag_template(const gchar *name, LogTemplate *template, LogMessage *msg, LogTemplateEvalOptions *options);
+
 #endif
